# Sample insert statements into TempTables. 
# There is no need to log changes to the Temp Tables.  
# Only changes to permanent tables need to be logged
# 
# 
# Shameema Esufali June 2020


# An example of loading census data from a spreadsheet is shown 
# at the end of the file.  You will need one spreadsheet for each TempTable.

/*
# Essential columns to insert 
# Note # HOM is essential only if non-standard. Any comments on the field sheets should be included.
# Tag must be a valid tag that matches a tag in the Tree Table, Stemtag must be a valid stemtag that 
# matches a stemtag in the Stem table which has the appropriate tree id for that tree. 
# Quadratname and Mnemonic may be omitted for old trees but are highly recommended for verification. 
# Codes must be a set of valid TSMcode values from TSMAttributes separated by semicolons eg "L;B"
# ExactDate must be in standard form "YYYY-MM-DD"
# Extra columns can be used to pass custom data such as tree height 
# or reference numbers. These must be passed into the permanent database
# by custom programs or they will be lost when the Temp tables are cleared out.
# X and Y Locations need to be passed in via CensusNewPlants or separately into
# TempLocations making sure that there is exactly one record in TempLocations 
# for each new tag. 

*/
-- **************************************************************************
#  Insert essential data 
-- **************************************************************************
 
INSERT INTO TempOldTrees 
 (Tag, StemTag, QuadratName,  Mnemonic,  PlotID
  PlotCensusNumber,  DBH,  HOM,  Codes,  Comments, ExactDate)
SELECT  Tag, StemTag, QuadratName,  Mnemonic,  PlotID,  
  PlotCensusNumber,  DBH,  HOM,  Codes,  Comments, ExactDate
 FROM CensusOldTrees_minimum;

INSERT INTO TempMultiStems ( Tag, StemTag,  PlotID
  PlotCensusNumber,  DBH,  HOM,  Codes,  Comments, ExactDate) 
SELECT Tag, StemTag,  PlotID
  PlotCensusNumber,  DBH,  HOM,  Codes,  Comments, ExactDate 
  FROM CensusMultiStems_minimum;

  
INSERT INTO TempNewPlants ( Tag, StemTag, QuadratName,  Mnemonic,  PlotID
  PlotCensusNumber,  DBH, X,Y, HOM,  Codes,  Comments, ExactDate) 
SELECT Tag, StemTag, QuadratName,  Mnemonic,  PlotID
  PlotCensusNumber,  DBH, X,Y, HOM,  Codes,  Comments, ExactDate  FROM CensusNewPlants;   

-- **************************************************************************
#  Create Location records
-- **************************************************************************
 
# Note the coordinates have to be quadrat coordinates. See how below. 

INSERT INTO TempLocations (QuadratName, Tag,  X, Y, PlotCensusNumber, CensusID, PlotID)
SELECT distinct QuadratName, Tag, qX, qY, PlotCensusNumber, CensusID, PlotID FROM TempNewPlant;

-- **************************************************************************
#  FINALLY Check that all new plants have coordinates and vice versa
-- **************************************************************************
 
# Check that the two tables have the same trees
SELECT COUNT(distinct tag) FROM TempLocations WHERE tag  IN (SELECT tag FROM TempNewPlants);
SELECT COUNT(distinct tag)  FROM TempNewPlants;   



-- Additional help below
-- **************************************************************************
# If your location data is not in quadrat coordinates adjust it
-- **************************************************************************

/*
1. Available coordinates are subquadrat coordinates.  Convert them into quadrat coordinates


2. Available coordinates are plot coordinates.  Convert them into quadrat Coordinates

3. Available coordinates are quadrat coordinates
   Make no adjustment.  Pass them in


Create a temporary table witht the structure below. 
+------------------+------------------+------+-----+---------+-------+
| Field            | Type             | Null | Key | Default | Extra |
+------------------+------------------+------+-----+---------+-------+
| QX               | float            | YES  |     | NULL    |       |
| QY              | float            | YES  |     | NULL    |       |
| QuadratID        | int(10) unsigned | YES  |     | NULL    |       |
| QuadratName      | varchar(12)      | YES  |     | NULL    |       |
| Tag              | varchar(10)      | YES  |     | NULL    |       |
| X                | float            | YES  |     | NULL    |       |
| Y                | float            | YES  |     | NULL    |       |
| PlotCensusNumber | int(10) unsigned | YES  |     | NULL    |       |
| CensusID         | int(10) unsigned | YES  |     | NULL    |       |
| PlotID           | int(10) unsigned | YES  |     | NULL    |       |
+------------------+------------------+------+-----+---------+-------+


 
1. In the example below, the assumptions are as follows
SX = Field measurement from the subquad origin

Subquad format is XY where 
 X is the number of 1m displacements of the subquad from the quadrat origin on the X axis
 Y is the number of 1m displacements of the subquad from the quadrat origin on the Y axis.  
 
Thus QX = (X) * 5 + SX
     QY = (X) * 5 + SY

     
IN MySQL this is coded as 
     QX = ((left(subquad,1))*5)+SX 
     QY = ((right(subquad,1))*5)+SY 
     PX = (left(lpad(a.quadratname,4,"0"),2))*20 + QX
     PY = (right(lpad(a.quadratname,4,"0"),2))*20 + QY
     
So the INSERT into TempLocations would be  
*/
INSERT INTO TempLocations (QuadratName, Tag,  X, Y, PlotCensusNumber, CensusID, PlotID)    
SELECT (
 QuadratName, Tag, PlotCensusNumber, CensusID, PlotID 
# Here X and Y are subquad coordinates < 5m 
 ((left(subquad,1))*5)+X QX, ((right(subquad,1))*5)+Y QY  
 )
 FROM CensusNewPlants;    
 
/*
2.  In this example X and Y are plot coordinates.
*/

# Subtract the coordinates of the quadrat origin to get the quadrat coords

DROP TABLE IF EXISTS TEMP;
CREATE TABLE TEMP (qx float(8),qy float(8)) 
 SELECT distinct quadratid, QuadratName, Tag, X, Y, PlotCensusNumber, CensusID, PlotID 
 FROM TempNewPlants;

UPDATE TempNewPlants a, Quadrat b SET a.quadratid=b.quadratid WHERE a.quadratname=b.quadratname; 

UPDATE TEMP a, Coordinates b  
 SET a.QX=a.X-b.PX , a.QY=a.Y-b.PY   
 WHERE a.quadratid=b.quadratid;
 
UPDATE TempNewPlants a, TEMP b
SET a.X=b.QX,a.y=b.Y
WHERE a.tag=b.tag
 

INSERT INTO TempLocations (QuadratName, Tag,  X, Y, PlotCensusNumber, CensusID, PlotID)
SELECT distinct QuadratName, Tag, qX, qY, PlotCensusNumber, CensusID, PlotID FROM TempNewPlants;

# DROP TABLE IF EXISTS TEMP;



-- **************************************************************************
# Full insert statements if you have all the columns in your spreadsheet.
-- **************************************************************************
 
INSERT INTO TempOldTrees ( TreeID, Tag, StemID, StemTag, QuadratID, QuadratName, Subquad, SpeciesID, Mnemonic, SubSpeciesID, PlotID, CensusID, PlotCensusNumber, PrimaryStem, X, Y, DBH, OldDBH, HOM, OldHOM, Codes, OldCodes, Comments, ExactDate, Errors, OldTrees, BigTrees, Extra1, Extra2, Extra3, Extra4, Extra5) 
SELECT TreeID, Tag, StemID, StemTag, QuadratID, QuadratName, Subquad, SpeciesID, Mnemonic, SubSpeciesID, PlotID, CensusID, PlotCensusNumber, PrimaryStem, X, Y, DBH, OldDBH, HOM, OldHOM, Codes, OldCodes, Comments, ExactDate, Errors, OldTrees, BigTrees, Extra1, Extra2, Extra3, Extra4, Extra5  FROM CensusOldTrees;

INSERT INTO TempMultiStems ( TreeID, Tag, StemID, StemTag, QuadratID, QuadratName, Subquad, SpeciesID, Mnemonic, SubSpeciesID, PlotID, CensusID, PlotCensusNumber, PrimaryStem, X, Y, DBH, OldDBH, HOM, OldHOM, Codes, OldCodes, Comments, ExactDate, Errors, OldTrees, BigTrees, Extra1, Extra2, Extra3, Extra4, Extra5) 
SELECT TreeID, Tag, StemID, StemTag, QuadratID, QuadratName, Subquad, SpeciesID, Mnemonic, SubSpeciesID, PlotID, CensusID, PlotCensusNumber, PrimaryStem, X, Y, DBH, OldDBH, HOM, OldHOM, Codes, OldCodes, Comments, ExactDate, Errors, OldTrees, BigTrees, Extra1, Extra2, Extra3, Extra4, Extra5 FROM CensusNewPlants;

INSERT INTO TempNewPlants ( TreeID, Tag, StemID, StemTag, QuadratID, QuadratName, Subquad, SpeciesID, Mnemonic, SubSpeciesID, PlotID, CensusID, PlotCensusNumber, PrimaryStem, X, Y, DBH, OldDBH, HOM, OldHOM, Codes, OldCodes, Comments, ExactDate, Errors, OldTrees, BigTrees, Extra1, Extra2, Extra3, Extra4, Extra5) 
SELECT TreeID, Tag, StemID, StemTag, QuadratID, QuadratName, Subquad, SpeciesID, Mnemonic, SubSpeciesID, PlotID, CensusID, PlotCensusNumber, PrimaryStem, X, Y, DBH, OldDBH, HOM, OldHOM, Codes, OldCodes, Comments, ExactDate, Errors, OldTrees, BigTrees, Extra1, Extra2, Extra3, Extra4, Extra5  FROM CensusNewPlants;



/*
-- **************************************************************************
SELECT "Loading data from  csv file with matching columns";
-- ************************************************************************** 
# Columns should be in the same order as the loading table.  The table shown below is a sample.
# Adapt it to your data.  You must include mandatory data for a successful upload. 

  
DROP TABLE IF EXISTS Census;
CREATE TABLE IF NOT EXISTS Census (
 plotname varchar(25),
 plotid int,
 family varchar(80),
 genus varchar(80),
 speciesname varchar(80),
 mnemonic varchar(10),
 subspecies varchar(80),
 speciesid int,
 subspeciesid int,
 quadratid int,
 px decimal(7,1),
 py decimal(7,1),
 qx decimal(7,1),
 qy decimal(7,1),
 sx decimal(7,1),
 sy decimal(7,1),
 treeid int,
 tag varchar(10),
 quadratname varchar(10),
 subquad varchar(10),
 stemid int,
 stemnumber int,
 stemtag varchar(10),
 primarystem varchar(10),
 censusid int,
 plotcensusnumber int,
 hom decimal(7,1),
 exactdate varchar(15),
 listoftsm varchar(15),
 dbh decimal(7,1),
 remarks varchar(150)
 );
 
 LOAD DATA LOCAL INFILE '../Census.csv' 
INTO TABLE Census fields OPTIONALLY ENCLOSED BY '"' TERMINATED BY ',' ENCLOSED BY '' IGNORE 1 LINES;
 
 */
