# Example of how to delete a tree from a CTFS database
# Shameema Esufali August 2020
#
# To delete trees
# Create an excel sheet with one column with a heading tag
# List all the tags you want to delete including other info to corroborate 
# Save it as a csv file from excel
# Replace the path and name of this file in the LOAD DATA line below
#
# Note: You need to create a Log record for every change to the database. 
# When deleting records, you must first create the log record so you
# do not lose information. 
#
-- **********************************************************************************************
-- Set up the database.  Replace the test database "blue" with the last frozen 
-- version of your database. Eg USE current_yoursite;  SOURCE ../bci_current_20August2020.sql;
DROP DATABASE IF EXISTS blue;
CREATE DATABASE IF NOT EXISTS blue;
USE blue;
SOURCE ../sampledatabases/blue.sql;


-- You must run the code below in your database.  Once you have run it for one script it is 
-- not necessary to run it for others but it will not cause a problem if you do. 
-- Do not remove the commands below. Change the path to reflect your directory path.
# 1. Installs a procedure that creates DBHAttributes from codes separated by a semicolon.
# The following columns must exist in the table tempinserttrees. 
# codes varchar(20),
# dbhid int,
# 2. Installs Log Procedures. 
SOURCE Procedures/LogProcedures.sql;
SOURCE Procedures/Utilities.sql;

# Some older databases have the old and new columns as NOT NULL
# the statement below corrects the error. 
ALTER TABLE Log MODIFY old VARCHAR(2048), MODIFY new VARCHAR(2048);
SELECT @n1:=count(*) FROM Log;

# This table is required in the database. 
DROP TABLE IF EXISTS TempLogFeeder;
CREATE TABLE TempLogFeeder(
  ChangedTableID int(10),
  ChangedColumn varchar(64),
  ChangeReason VARCHAR(256)
  );
  
-- end database setup **********************************************************************  

# *******************************************************************************************************
# ***** LOAD IN THE TAGS YOU WANT TO DELETE 
# *******************************************************************************************************
# CHANGE TO APPROPRIATE VALUES FOR YOUR DATA.  PersonnelID is the primary key 
# from the Personnel table 
# for the person who is responsible for authenticating the change, 
SET  @personnelid=1, @action="D";

DROP TABLE IF EXISTS tempdeletetrees;
CREATE TABLE tempdeletetrees (
Tag char(15),
Mnemonic char(15),
Quadratname char(10),
Changereason VARCHAR(128)
)
ENGINE=INNODB;

LOAD DATA LOCAL INFILE '../datafiles/deletetrees.csv' INTO TABLE tempdeletetrees 
 FIELDS OPTIONALLY ENCLOSED BY '"' terminated by ',' IGNORE 1 LINES;

ALTER TABLE tempdeletetrees ADD COLUMN treeid int;

# You can remove the trim function to speed this up for large
# tables if there is no mismatch between the data file and the database. 
# eg  WHERE a.tag=b.tag;

UPDATE tempdeletetrees a,ViewFullTable b SET a.treeid=b.treeid 
 WHERE TRIM(LEADING '0' FROM a.tag)=TRIM(LEADING '0' FROM b.tag);

# *******************************************************************************************************
# ***** FIND THE CORRESPONDING STEMS 
# *******************************************************************************************************

DROP TABLE IF EXISTS tempdeletestems;

CREATE TABLE tempdeletestems AS
 SELECT a.tag,b.treeid,b.stemid,a.changereason
 FROM tempdeletetrees a,ViewFullTable b
 WHERE a.treeid=b.treeid group by a.tag,b.treeid,b.stemid;

# *******************************************************************************************************
# ***** FIND THE CORRESPONDING DBH RECORDS 
# *******************************************************************************************************

DROP TABLE IF EXISTS tempdeletedbh;
CREATE TABLE tempdeletedbh AS
 SELECT a.tag,b.treeid,b.stemid,b.dbhid,a.changereason 
 FROM tempdeletetrees a,ViewFullTable b
 WHERE a.treeid=b.treeid;


# *******************************************************************************************************
# ***** CREATE LOG RECORDS FOR EACH TABLE YOU ARE DELETING FROM 
# # Note: You should be able to use this code without any changes.
# *******************************************************************************************************

# Log record for Trees
TRUNCATE TempLogFeeder;
INSERT INTO TempLogFeeder (ChangedTableID,ChangedColumn,ChangeReason) SELECT distinct TreeID,"ALL",changereason FROM tempdeletetrees;
CALL logRecordSet(@personnelid,"Tree",@action) ; 

# Log record for Stems
TRUNCATE TempLogFeeder;
INSERT INTO TempLogFeeder (ChangedTableID,ChangedColumn,ChangeReason) SELECT  StemID,"ALL",changereason FROM tempdeletestems; 
CALL logRecordSet(@personnelid,"Stem",@action); 

# Log record for DBH records
TRUNCATE TempLogFeeder;
INSERT INTO TempLogFeeder (ChangedTableID,ChangedColumn,ChangeReason) SELECT  DBHID,"ALL",changereason FROM tempdeletedbh;
CALL logRecordSet(@personnelid,"DBH",@action); 


# Log record for DBHattributes records.
TRUNCATE TempLogFeeder;
INSERT INTO TempLogFeeder (ChangedTableID,ChangedColumn,ChangeReason) SELECT DBHAttID,"ALL","reason for inserting these records" FROM DBHAttributes  WHERE DBHID in (SELECT DBHID FROM tempdeletedbh);
CALL logRecordSet(@personnelid,"DBHAttributes",@action); 


# *******************************************************************************************************
# ***** NOW DELETE ALL RECORDS BEGINNING WITH THE ATTRIBUTES
# *******************************************************************************************************

DELETE FROM DBHAttributes WHERE dbhid IN (SELECT DBHID FROM tempdeletedbh);
DELETE FROM DBH WHERE dbhid IN (SELECT DBHID FROM tempdeletedbh);
DELETE FROM Stem WHERE stemid IN (SELECT stemid FROM tempdeletestems);
DELETE FROM Tree WHERE treeid IN (SELECT distinct treeid FROM tempdeletestems);


# *******************************************************************************************************
# Rebuild Views. Replace the first parameter with your database name 
# *******************************************************************************************************

CALL ctfsweb_webuser.CreateViewTaxonomy("blue","'foo'");
CALL ctfsweb_webuser.CreateFullView( "blue","ViewFullTable");

# *******************************************************************************************************
# ***** FINALLY CLEAN UP THE DATABASE BY REMOVING ALL THE TEMP TABLES YOU CREATED
# *******************************************************************************************************

# DROP TABLE tempdeletedbh;
# DROP TABLE tempdeletestems;
# DROP TABLE tempdeletetrees;




